@@ -162,6 +162,64 @@ void mtm_test_construction()
         assert(mtx.get_type(1,4) == mtm::element_string);
         assert(mtx.get_string(1,4) == "foo");
     }
+
+    {
+        // construct with an array of data.
+        vector<double> vals;
+        vals.push_back(1.1);
+        vals.push_back(1.2);
+        vals.push_back(1.3);
+        vals.push_back(1.4);
+        mtx_type mtx(2, 2, vals.begin(), vals.end());
+        mtx_type::size_pair_type sz = mtx.size();
+        assert(sz.row == 2 && sz.column == 2);
+        assert(mtx.get_numeric(0,0) == 1.1);
+        assert(mtx.get_numeric(1,0) == 1.2);
+        assert(mtx.get_numeric(0,1) == 1.3);
+        assert(mtx.get_numeric(1,1) == 1.4);
+
+        try
+        {
+            mtx_type mtx2(3, 2, vals.begin(), vals.end());
+            assert(!"Construction of this matrix should have failed!");
+        }
+        catch (const invalid_arg_error& e)
+        {
+            // Good.
+            cout << "exception caught (as expected) which says: " << e.what() << endl;
+        }
+
+        try
+        {
+            // Trying to initialize a matrix with array of unsupported data
+            // type should end with an exception thrown.
+            vector<size_t> vals_ptr(4, 22);
+            mtx_type mtx3(2, 2, vals_ptr.begin(), vals_ptr.end());
+            assert(!"Construction of this matrix should have failed!");
+        }
+        catch (const exception& e)
+        {
+            cout << "exception caught (as expected) which says: " << e.what() << endl;
+        }
+    }
+
+    {
+        // Construct with an array of custom string type.
+        vector<custom_string> vals;
+        vals.push_back(custom_string("A"));
+        vals.push_back(custom_string("B"));
+        vals.push_back(custom_string("C"));
+        vals.push_back(custom_string("D"));
+        mtx_custom_type mtx(1, 4, vals.begin(), vals.end());
+        assert(mtx.get_string(0,0).get() == "A");
+        assert(mtx.get_string(0,1).get() == "B");
+        assert(mtx.get_string(0,2).get() == "C");
+        assert(mtx.get_string(0,3).get() == "D");
+        assert(mtx.get_type(0,0) == mtm::element_string);
+        assert(mtx.get_type(0,1) == mtm::element_string);
+        assert(mtx.get_type(0,2) == mtm::element_string);
+        assert(mtx.get_type(0,3) == mtm::element_string);
+    }
 }
 
 void mtm_test_data_insertion()
@@ -277,6 +335,38 @@ void mtm_test_set_empty()
         assert(mtx.get_type(2, 3) != mtm::element_empty);
         assert(mtx.get_type(2, 4) != mtm::element_empty);
     }
+
+    {
+        // Set a range of elements empty.
+        mtx_type mtx(5, 3, string("A"));
+        cout << "setting element (0,1) to (1,2) empty..." << endl;
+        mtx.set_empty(1, 0, 6); // rows 1-4 in column 0 and rows 0-1 in column 1.
+        assert(mtx.get_type(0, 0) == mtm::element_string);
+        assert(mtx.get_type(1, 0) == mtm::element_empty);
+        assert(mtx.get_type(2, 0) == mtm::element_empty);
+        assert(mtx.get_type(3, 0) == mtm::element_empty);
+        assert(mtx.get_type(4, 0) == mtm::element_empty);
+        assert(mtx.get_type(0, 1) == mtm::element_empty);
+        assert(mtx.get_type(1, 1) == mtm::element_empty);
+        assert(mtx.get_type(2, 1) == mtm::element_string);
+        assert(mtx.get_type(3, 1) == mtm::element_string);
+        assert(mtx.get_type(4, 1) == mtm::element_string);
+        assert(mtx.get_type(0, 2) == mtm::element_string);
+        assert(mtx.get_type(1, 2) == mtm::element_string);
+        assert(mtx.get_type(2, 2) == mtm::element_string);
+        assert(mtx.get_type(3, 2) == mtm::element_string);
+        assert(mtx.get_type(4, 2) == mtm::element_string);
+
+        try
+        {
+            mtx.set_empty(2, 2, 0);
+            assert(false);
+        }
+        catch (const std::exception&)
+        {
+            cout << "exception thrown on length of zero as expected" << endl;
+        }
+    }
 }
 
 void mtm_test_swap()
@@ -502,21 +592,27 @@ public:
             case mtm::element_boolean:
             {
                 mtx_type::boolean_block_type::const_iterator it = mtx_type::boolean_block_type::begin(*node.data);
-                mtx_type::boolean_block_type::const_iterator it_end = mtx_type::boolean_block_type::end(*node.data);
+                std::advance(it, node.offset);
+                mtx_type::boolean_block_type::const_iterator it_end = it;
+                std::advance(it_end, node.size);
                 std::for_each(it, it_end, print_element<bool>());
             }
             break;
             case mtm::element_string:
             {
                 mtx_type::string_block_type::const_iterator it = mtx_type::string_block_type::begin(*node.data);
-                mtx_type::string_block_type::const_iterator it_end = mtx_type::string_block_type::end(*node.data);
+                std::advance(it, node.offset);
+                mtx_type::string_block_type::const_iterator it_end = it;
+                std::advance(it_end, node.size);
                 std::for_each(it, it_end, print_element<mtx_type::string_type>());
             }
             break;
             case mtm::element_numeric:
             {
                 mtx_type::numeric_block_type::const_iterator it = mtx_type::numeric_block_type::begin(*node.data);
-                mtx_type::numeric_block_type::const_iterator it_end = mtx_type::numeric_block_type::end(*node.data);
+                std::advance(it, node.offset);
+                mtx_type::numeric_block_type::const_iterator it_end = it;
+                std::advance(it_end, node.size);
                 std::for_each(it, it_end, print_element<double>());
             }
             break;
@@ -546,6 +642,34 @@ void mtm_test_walk()
     mtx.walk(func);
 }
 
+void mtm_test_walk_subset()
+{
+    {
+        stack_printer __stack_printer__("::mtm_test_walk_subset test1");
+        mtx_type mtx(4, 4); // single column matrix to make it easier.
+        mtx.set(1, 1, 1.1);
+        mtx.set(2, 1, 1.2);
+        mtx.set(3, 1, 1.3);
+        mtx.set(0, 2, string("A1"));
+        mtx.set(1, 2, string("A2"));
+        mtx.set(2, 2, false);
+        walk_element_block func;
+        mtx.walk(func, mtx_type::size_pair_type(1,1), mtx_type::size_pair_type(2, 2));
+    }
+    {
+        stack_printer __stack_printer__("::mtm_test_walk_subset test2");
+        mtx_type mtx(4, 4); // single column matrix to make it easier.
+        mtx.set(0, 1, 1.0);
+        mtx.set(1, 1, 1.1);
+        mtx.set(0, 2, string("A1"));
+        mtx.set(1, 2, string("A2"));
+        mtx.set(2, 2, string("A3"));
+        mtx.set(3, 2, string("A4"));
+        walk_element_block func;
+        mtx.walk(func, mtx_type::size_pair_type(1,1), mtx_type::size_pair_type(2, 2));
+    }
+}
+
 void mtm_test_custom_string()
 {
     stack_printer __stack_printer__("::mtm_test_custom_string");
@@ -604,6 +728,69 @@ void mtm_test_position()
     mtx.set(pos, string("ABC"));
     assert(mtx.get_type(2, 1) == mtm::element_string);
     assert(mtx.get_string(2, 1) == "ABC");
+
+    // Start over, and test the traversal of position object.
+    pos = mtx.position(0, 0);
+    mtx_type::size_pair_type mtx_pos = mtx.matrix_position(pos);
+    assert(mtx_pos.column == 0);
+    assert(mtx_pos.row == 0);
+
+    pos = mtx_type::next_position(pos);
+    mtx_pos = mtx.matrix_position(pos);
+    assert(mtx_pos.column == 0);
+    assert(mtx_pos.row == 1);
+
+    pos = mtx_type::next_position(pos);
+    mtx_pos = mtx.matrix_position(pos);
+    assert(mtx_pos.column == 0);
+    assert(mtx_pos.row == 2);
+
+    pos = mtx_type::next_position(pos);
+    mtx_pos = mtx.matrix_position(pos);
+    assert(mtx_pos.column == 1);
+    assert(mtx_pos.row == 0);
+
+    pos = mtx_type::next_position(pos);
+    mtx_pos = mtx.matrix_position(pos);
+    assert(mtx_pos.column == 1);
+    assert(mtx_pos.row == 1);
+
+    pos = mtx_type::next_position(pos);
+    mtx_pos = mtx.matrix_position(pos);
+    assert(mtx_pos.column == 1);
+    assert(mtx_pos.row == 2);
+
+    pos = mtx_type::next_position(pos);
+    assert(pos == mtx.end_position());
+}
+
+void mtm_test_set_data_via_position()
+{
+    stack_printer __stack_printer__("::mtm_test_set_data_via_position");
+    mtx_type mtx(5, 4);
+    mtx_type::position_type pos = mtx.position(0, 1);
+    vector<double> data;
+    data.push_back(1.1);
+    data.push_back(1.2);
+    data.push_back(1.3);
+    data.push_back(1.4);
+    data.push_back(1.5);
+    pos = mtx.set(pos, data.begin(), data.end());
+    assert(mtx.get<double>(0, 1) == 1.1);
+    assert(mtx.get<double>(1, 1) == 1.2);
+    assert(mtx.get<double>(2, 1) == 1.3);
+    assert(mtx.get<double>(3, 1) == 1.4);
+    assert(mtx.get<double>(4, 1) == 1.5);
+
+    mtx_type::size_pair_type mtx_pos = mtx.matrix_position(pos);
+    assert(mtx_pos.row == 0);
+    assert(mtx_pos.column == 1);
+    pos = mtx.position(pos, 0, 2);
+    pos = mtx.set(pos, string("test"));
+    pos = mtx_type::next_position(pos);
+    pos = mtx.set(pos, true);
+    assert(mtx.get<string>(0, 2) == "test");
+    assert(mtx.get<bool>(1, 2) == true);
 }
 
 /**
@@ -776,34 +963,83 @@ void mtm_perf_test_iterate_elements()
     }
 }
 
-int main (int argc, char **argv)
+void mtm_perf_test_insert_via_position_object()
 {
-    cmd_options opt;
-    if (!parse_cmd_options(argc, argv, opt))
-        return EXIT_FAILURE;
-
-    if (opt.test_func)
+    size_t rowsize = 3000, colsize = 3000;
     {
-        mtm_test_construction();
-        mtm_test_data_insertion();
-        mtm_test_data_insertion_multiple();
-        mtm_test_set_empty();
-        mtm_test_swap();
-        mtm_test_transpose();
-        mtm_test_resize();
-        mtm_test_copy();
-        mtm_test_assignment();
-        mtm_test_numeric();
-        mtm_test_walk();
-        mtm_test_custom_string();
-        mtm_test_position();
+        stack_printer __stack_printer__("::mtm_perf_test_insert_via_position_object (column and row positions)");
+        mtx_type mx(rowsize, colsize);
+        for (size_t i = 0; i < rowsize; ++i)
+        {
+            for (size_t j = 0; j < colsize; ++j)
+            {
+                mx.set(i, j, 1.1);
+            }
+        }
     }
 
-    if (opt.test_perf)
     {
-        mtm_perf_test_storage_creation();
-        mtm_perf_test_storage_set_numeric();
-        mtm_perf_test_iterate_elements();
+        stack_printer __stack_printer__("::mtm_perf_test_insert_via_position_object (position object)");
+        mtx_type mx(rowsize, colsize);
+        mtx_type::position_type pos = mx.position(0, 0);
+        for (size_t i = 0; i < rowsize; ++i)
+        {
+            for (size_t j = 0; j < colsize; ++j)
+            {
+                pos = mx.set(pos, 1.1);
+                pos = mtx_type::next_position(pos);
+            }
+        }
+    }
+
+    {
+        stack_printer __stack_printer__("::mtm_perf_test_insert_via_position_object (position object)");
+        mtx_type mx(rowsize, colsize);
+        mtx_type::position_type pos = mx.position(0, 0);
+        for (; pos != mx.end_position(); pos = mtx_type::next_position(pos))
+            pos = mx.set(pos, 1.1);
+    }
+}
+
+int main (int argc, char **argv)
+{
+    try
+    {
+        cmd_options opt;
+        if (!parse_cmd_options(argc, argv, opt))
+            return EXIT_FAILURE;
+
+        if (opt.test_func)
+        {
+            mtm_test_construction();
+            mtm_test_data_insertion();
+            mtm_test_data_insertion_multiple();
+            mtm_test_set_empty();
+            mtm_test_swap();
+            mtm_test_transpose();
+            mtm_test_resize();
+            mtm_test_copy();
+            mtm_test_assignment();
+            mtm_test_numeric();
+            mtm_test_walk();
+            mtm_test_walk_subset();
+            mtm_test_custom_string();
+            mtm_test_position();
+            mtm_test_set_data_via_position();
+        }
+
+        if (opt.test_perf)
+        {
+            mtm_perf_test_storage_creation();
+            mtm_perf_test_storage_set_numeric();
+            mtm_perf_test_iterate_elements();
+            mtm_perf_test_insert_via_position_object();
+        }
+    }
+    catch (const std::exception& e)
+    {
+        fprintf(stdout, "Test failed: %s\n", e.what());
+        return EXIT_FAILURE;
     }
 
     cout << "Test finished successfully!" << endl;
